from os import environ
from pydoc import cli
from typing import Union
from dotenv import load_dotenv
from threading import Timer
from io import BytesIO, TextIOWrapper


class Slots:
    @property
    def __dict__(self):
        return {s: getattr(self, s) for s in self.__slots__ if hasattr(self, s)}


def parse(cls, val, default=None):
    try:
        return cls(val)
    except:
        return default


def load_env_s3(
    s3bucket: str = environ.get("ENV_S3_BUCKET"),
    s3path: str = environ.get("ENV_S3_KEY"),
    refresh: Union[int, float, str, None] = environ.get("ENV_S3_REFRESH", 0),
    s3client=None,
    *arg,
    **kw
):
    if not (("ENV_S3_BUCKET" in environ) or ("ENV_S3_KEY" in environ)):
        return False

    if s3client is None:
        from boto3 import client

        s3client = client("s3")

    if refresh is None:
        refresh = 0
    elif isinstance(refresh, str):
        refresh = float(refresh)

    if refresh > 0:
        t = Timer(
            interval=refresh,
            function=load_env_s3,
            args=[s3bucket, s3path, refresh, s3client] + list(arg),
            kwargs=kw,
        )
        t.start()

    with BytesIO() as fp:
        s3client.download_fileobj(s3bucket, s3path, fp)
        fp.seek(0)
        return load_dotenv(stream=TextIOWrapper(fp, "utf-8"), *arg, **kw)
